@@ -98,20 +98,14 @@ int main(int argc, char *argv[])
 
     KCrash::initialize();
 
-    // If this flag gets set then JuK will quit if you click the cover on the track
-    // announcement popup when JuK is only in the system tray (the systray has no widget).
-
-    a.setQuitOnLastWindowClosed(false);
-
     // Create the main window and such
-
     JuK *juk = new JuK(parser.positionalArguments());
 
     if(a.isSessionRestored() && KMainWindow::canBeRestored(1))
         juk->restore(1, false /* don't show */);
 
     KConfigGroup config(KSharedConfig::openConfig(), "Settings");
-    if(!config.readEntry("StartDocked", false)) {
+    if(!config.readEntry("StartDocked", false) || !config.readEntry("DockInSystemTray", false)) {
         juk->show();
     }
     else if(!a.isSessionRestored()) {
