@@ -1,6 +1,6 @@
 /**
  * Copyright (C) 2002-2004 Scott Wheeler <wheeler@kde.org>
- * Copyright (C) 2008 Michael Pyne <mpyne@kde.org>
+ * Copyright (C) 2008-2018 Michael Pyne <mpyne@kde.org>
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -35,9 +35,11 @@
 #include <QCursor>
 #include <QDir>
 #include <QDirIterator>
+#include <QHash>
 #include <QToolTip>
 #include <QFile>
 #include <QFileDialog>
+#include <QHeaderView>
 #include <QResizeEvent>
 #include <QMouseEvent>
 #include <QKeyEvent>
@@ -51,6 +53,8 @@
 #include <QPixmap>
 #include <QStackedWidget>
 #include <QScrollBar>
+#include <QPainter>
+
 #include <id3v1genres.h>
 
 #include <time.h>
@@ -92,10 +96,6 @@ static bool manualResize()
     return action<KToggleAction>("resizeColumnsManually")->isChecked();
 }
 
-/**
- * A tooltip specialized to show full filenames over the file name column.
- */
-
 ////////////////////////////////////////////////////////////////////////////////
 // static members
 ////////////////////////////////////////////////////////////////////////////////
@@ -229,7 +229,7 @@ QString Playlist::name() const
 
 FileHandle Playlist::currentFile() const
 {
-    return playingItem() ? playingItem()->file() : FileHandle::null();
+    return playingItem() ? playingItem()->file() : FileHandle();
 }
 
 int Playlist::time() const
@@ -289,7 +289,7 @@ void Playlist::playNext()
 void Playlist::stop()
 {
     m_history.clear();
-    setPlaying(0);
+    setPlaying(nullptr);
 }
 
 void Playlist::playPrevious()
@@ -469,6 +469,9 @@ void Playlist::setSearchEnabled(bool enabled)
         setItemsVisible(items(), true);
 }
 
+// Mostly seems to be for DynamicPlaylist
+// TODO: See if this can't all be eliminated by making 'is-playing' a predicate
+// of the playlist item itself
 void Playlist::synchronizePlayingItems(const PlaylistList &sources, bool setMaster)
 {
     foreach(const Playlist *p, sources) {
@@ -773,6 +776,39 @@ void Playlist::removeFromDisk(const PlaylistItemList &items)
     }
 }
 
+void Playlist::synchronizeItemsTo(const PlaylistItemList &itemList)
+{
+    const auto &existingItems = items();
+    if(qAbs(itemList.count() - existingItems.count()) >
+        qMax(itemList.count(), existingItems.count()) / 2)
+    {
+        // Large imbalance in list sizes, just clear all and add without
+        // figuring out the diff also
+        clearItems(existingItems);
+        createItems(itemList);
+        return;
+    }
+
+    // Determine differences between existing playlist items and patch up
+    QHash<CollectionListItem *, PlaylistItem *> oldItems;
+    oldItems.reserve(qMax(existingItems.count(), itemList.count()));
+
+    for(const auto &item : existingItems) {
+        oldItems.insert(item->collectionItem(), item);
+    }
+
+    PlaylistItemList newItems;
+
+    for(const auto &item : itemList) {
+        if(oldItems.remove(item->collectionItem()) == 0) {
+            newItems.append(item->collectionItem());
+        }
+    }
+
+    clearItems(PlaylistItemList(oldItems.values()));
+    createItems(newItems);
+}
+
 void Playlist::dragEnterEvent(QDragEnterEvent *e)
 {
     if(CoverDrag::isCover(e->mimeData())) {
@@ -859,6 +895,9 @@ void Playlist::keyPressEvent(QKeyEvent *event)
         }
 
     }
+    else if(event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter) {
+        slotPlayCurrent();
+    }
 
     QTreeWidget::keyPressEvent(event);
 }
@@ -1003,7 +1042,7 @@ void Playlist::read(QDataStream &s)
         if(file.isEmpty())
             throw BICStreamException();
 
-        after = createItem(FileHandle(file), after, false);
+        after = createItem(FileHandle(file), after);
     }
 
     m_blockDataChanged = false;
@@ -1036,6 +1075,25 @@ void Playlist::resizeEvent(QResizeEvent *re)
     QTreeWidget::resizeEvent(re);
 }
 
+// Reimplemented to show a visual indication of which of the view's playlist
+// items is actually playing.
+void Playlist::drawRow(QPainter *p, const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+    PlaylistItem *item = static_cast<PlaylistItem *>(itemFromIndex(index));
+    if(Q_LIKELY(!PlaylistItem::playingItems().contains(item))) {
+        return QTreeWidget::drawRow(p, option, index);
+    }
+
+    // Seems that the view draws the background now so we have to do this
+    // manually
+    p->fillRect(option.rect, QPalette{}.midlight());
+
+    QStyleOptionViewItem newOption {option};
+    newOption.font.setBold(true);
+
+    QTreeWidget::drawRow(p, newOption, index);
+}
+
 void Playlist::insertItem(QTreeWidgetItem *item)
 {
     // Because we're called from the PlaylistItem ctor, item may not be a
@@ -1062,10 +1120,9 @@ void Playlist::addColumn(const QString &label, int)
     setHeaderLabels(m_columns);
 }
 
-PlaylistItem *Playlist::createItem(const FileHandle &file,
-                                   QTreeWidgetItem *after, bool emitChanged)
+PlaylistItem *Playlist::createItem(const FileHandle &file, QTreeWidgetItem *after)
 {
-    return createItem<PlaylistItem>(file, after, emitChanged);
+    return createItem<PlaylistItem>(file, after);
 }
 
 void Playlist::createItems(const PlaylistItemList &siblings, PlaylistItem *after)
@@ -1284,9 +1341,6 @@ void Playlist::slotInitialize()
             this, SLOT(slotInlineEditDone(QTreeWidgetItem*,QString,int)));*/
     connect(this, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)),
             this, SLOT(slotPlayCurrent()));
-    // FIXME returnPressed
-    /*connect(this, SIGNAL(returnPressed(QTreeWidgetItem*)),
-            this, SLOT(slotPlayCurrent()));*/
 
     // FIXME rename
     /*connect(renameLineEdit(), SIGNAL(completionModeChanged(KCompletion::CompletionMode)),
@@ -1332,14 +1386,14 @@ void Playlist::setDynamicListsFrozen(bool frozen)
 
 CollectionListItem *Playlist::collectionListItem(const FileHandle &file)
 {
-    if(!QFile::exists(file.absFilePath())) {
-        qCCritical(JUK_LOG) << "File" << file.absFilePath() << "does not exist.";
-        return 0;
-    }
-
     CollectionListItem *item = CollectionList::instance()->lookup(file.absFilePath());
 
     if(!item) {
+        if(!QFile::exists(file.absFilePath())) {
+            qCCritical(JUK_LOG) << "File" << file.absFilePath() << "does not exist.";
+            return nullptr;
+        }
+
         item = CollectionList::instance()->createItem(file);
     }
 
@@ -1394,6 +1448,7 @@ void Playlist::setup()
 {
     setRootIsDecorated(false);
     setContextMenuPolicy(Qt::CustomContextMenu);
+    setUniformRowHeights(true);
 
     connect(header(), SIGNAL(sectionMoved(int,int,int)), this, SLOT(slotColumnOrderChanged(int,int,int)));
 
@@ -1433,10 +1488,8 @@ void Playlist::loadFile(const QString &fileName, const QFileInfo &fileInfo)
 
     setSortingEnabled(false);
 
-    PlaylistItem *after = 0;
-
+    PlaylistItem *after = nullptr;
     m_disableColumnWidthUpdates = true;
-
     m_blockDataChanged = true;
 
     while(!stream.atEnd()) {
@@ -1450,17 +1503,16 @@ void Playlist::loadFile(const QString &fileName, const QFileInfo &fileInfo)
         if(item.exists() && item.isFile() && item.isReadable() &&
            MediaFiles::isMediaFile(item.fileName()))
         {
-            after = createItem(FileHandle(item), after, false);
+            after = createItem(FileHandle(item), after);
         }
     }
 
     m_blockDataChanged = false;
+    m_disableColumnWidthUpdates = false;
 
     file.close();
 
     playlistItemsChanged();
-
-    m_disableColumnWidthUpdates = false;
 }
 
 void Playlist::setPlaying(PlaylistItem *item, bool addToHistory)
@@ -1612,7 +1664,7 @@ void Playlist::addFile(const QString &file, FileHandleList &files, bool importPl
         while(dirIterator.hasNext()) {
             // We set importPlaylists to the value from the add directories
             // dialog as we want to load all of the ones that the user has
-            // explicitly asked for, but not those that we find in toLower
+            // explicitly asked for, but not those that we find in lower
             // directories.
 
             addFile(dirIterator.next(), files,
@@ -1639,8 +1691,10 @@ void Playlist::addFileHelper(FileHandleList &files, PlaylistItem **after, bool i
         if(visible)
             m_collection->raiseDistraction();
 
+        PlaylistItem *newAfter = *after;
         foreach(const FileHandle &fileHandle, files)
-            *after = createItem(fileHandle, *after, false);
+            newAfter = createItem(fileHandle, newAfter);
+        *after = newAfter;
 
         files.clear();
 
